import os
import sys

import soya
from soya import pudding

from style import Style

import config


class Application:
  """ Main application object """
  state_stack = []

  def _get_current_state(self):
    try:
      return self.state_stack[-1]
    except IndexError:
      return None

  state = property(_get_current_state)

  def __init__(self):
    soya.path.append(os.path.join(os.path.dirname(__file__), 'data'))

    self.config = config.Config()

    self.fullscreen = self.config.getboolean('video', 'fullscreen')

    if '--help' in sys.argv:
      print """
Overflow help:
  --fullscreen | -fs        :  use fullscreen
  --no-fullscreen | -nfs    :  dont use fullscreen
  --dump-config             :  writes the default congif file
"""
      sys.exit()
    
    if '--dump-config' in sys.argv:
      print "dumping config"
      self.config.save()
      sys.exit()

    if '--update-mesh' in sys.argv:
      print "converting meshes"
      soya.Cal3dShape.get('Green')
      soya.Cal3dShape.get('Purple')
      soya.Cal3dShape.get('Blue')
      soya.Cal3dShape.get('Red')
      soya.Cal3dShape.get('Orange')
      soya.Cal3dShape.get('Yellow')
      sys.exit()

    if '--fullscreen' in sys.argv or '-fs' in sys.argv:
      self.fullscreen = True
    elif '--no-fullscreen' in sys.argv or '-nfs' in sys.argv:
      self.fullscreen = False

    sound = None
    
    if self.config.getboolean('audio', 'try_openal'):
      try:
        import soya.openal4soya as sound
      except ImportError:
        print "Can not load openal"
        sound = None

    if not sound and self.config.getboolean('audio', 'try_sdlmixer'):
      try:
        import soya.sdl_mixer4soya as sound
      except ImportError:
        print "Can not load sdlmixer"
        sound = None

    print "Using sound: %s" % sound
    self.sound = sound
    self.music = None
    self.music_file = ''

  def init_soya(self):
    """ intialize soya specific data """
  
    width = self.config.getint('video', 'width')
    height = self.config.getint('video', 'height')

    soya.init(title = "Overflow", width = width, height = height, fullscreen = self.fullscreen)
    pudding.init(style = Style())

  def set_state(self, state, stack = False, **kwargs):
    """ pass a state class to set current app state. Pass an instance or 
    class type. Use an instance to resume a state and class type to start
    a new state """

    # make sure we end up with an instance of the state
    if type(state) is type:
      instance = state(self, **kwargs)
    else:
      instance = state
      instance.application = self

    if self.sound:
      self.sound.init(instance.camera)    

    # stop the current state
    if self.state is not None and not stack:
      print "Stopping state: %s" % self.state
      self.state.stop()

      # pop the current state from the stack if required
      if not stack:
        self.state_stack.pop()
      
    # append the new state on the stack
    self.state_stack.append(instance)
    print "Starting state: %s" % self.state
    self.state.start()

  def pop_state(self):
    if self.state:
      print "Stopping state: %s" % self.state
      self.state.stop()

    self.state_stack.pop()

    if self.state:
      print "Continueing state: %s" % self.state

  def run(self):
    """ Actually start the application """
   
    if self.state is None:
      raise "No state set!"

    self.state.start()

  def play_music(self, file):
    if not self.sound or self.music_file == file:
      return 

    if self.music:
      self.music.stop()

    self.music = self.sound.play(file, looping = 1, gain = 0.1)
    self.music_file = file

  def play_effect(self, file):
    if not self.sound:
      return 
    self.sound.play(file, gain = 2.0) 


