import soya
import soya.cube

from utils import dist
from board.chargeball import ChargeBall

class Character(soya.World):
  """
  Character class. This class should be controlled by a Player class:
  LocalPlayer, AIPlayer, NetworkPlayer
  """
  def __init__(self, parent, id):
    soya.World.__init__(self, parent)
    self.create_shape(id)
    self.move_speed = 2.0
    self.throw_speed = 2.0
    self.status = 'idle'
    self.move_vector = soya.Vector(self, 0.0, 0.0, 0.0)
    self.throw_vector = soya.Vector(self, 0.0, 0.0, 0.0)

    self.solid = 0

  def create_shape(self, id):
    self.suit = soya.Volume(self, soya.Shape.get('Purple'))
    self.suit.solid = 0

  def advance_time(self, proportion):
    if self.status == 'moving':
      self.add_mul_vector(proportion * 0.03 * self.move_speed, self.move_vector)
    if self.status == 'throwing':
      self.add_mul_vector(proportion * 0.03 * self.throw_speed, self.move_vector)
      #other character animation
  
  # This is just a puppet. We shouldn't implement begin_round in this class

class Player:
    def __init__(self, char, state):
        self.char = char
        self.state = state
        self.board = state.board
        self.status = 'idle'
        self.speed = 2.0
        self.throw_speed = 2.0
        self.position = (0.0, 0.0)
        self.chargeball_val = 0
        self.power = 100
        self.powerinc = 1 # increment per round
        self.game_over = False
        self.fallspeed = 0.0

        # the source of the throw
        self.throw_src = None
    
    def get_new_pos(self, dest):
        x = self.position[0]
        z = self.position[1]
        p1 = soya.Point()
        p2 = soya.Point()
        p1.x = x
        p1.z = z
        p2.x = dest[0]
        p2.z = dest[1]
        v = p1.vector_to(p2)
        v.normalize()
        v = v * (self.speed * 0.1)
        x = x + v.x
        z = z + v.z
        return (x, z)

    def get_new_throw_pos(self):
        x = self.position[0]
        z = self.position[1]
        p1 = soya.Point()
        p2 = soya.Point()
        p1.x = self.throw_src.gpoint[0]
        p1.z = self.throw_src.gpoint[1]
        p2.x = x
        p2.z = z
        v = p1.vector_to(p2)
        v.normalize()
        v = v * (self.throw_speed * 0.1)
        x = x + v.x
        z = z + v.z
        return (x, z)

    def sync_char(self):
        self.char.x = self.position[0]
        self.char.z = self.position[1]

    def set_position(self, newpos):
        self.position = newpos
        self.char.x = newpos[0]
        self.char.z = newpos[1]

    def get_position(self):
        return self.position


class HumanPlayer(Player):
    def __init__(self, char, state):
        Player.__init__(self, char, state)

        # last visited polygon
        self.last_poly = None
        
        # the position on the path
        self.path_position = 0

        self.path = []
        self.char.begin_round = self.begin_round

    def begin_round(self):
        self.power += self.powerinc
        if self.power > 100:
          self.power = 100
        #curpoly = self.board.picking_context.raypick(self.char,
        #                                     self.state.camera.vector_to(self.char))
        p = soya.Point(self.board);p.x=self.char.x;p.y=0.1;p.z=self.char.z
        v = soya.Vector(self.char, 0.0, -1.0, 0.0)
        picres = self.board.raypick_polys(p, v)
        if picres:
          curpoly = picres[0]
          curpolypos = curpoly.gpoint
        else:
          curpoly = None
          curpolypos = (0.0, 0.0)

        if self.status == 'moving':
          d = dist(self.position, curpolypos)
          for p in self.path[self.path_position:]:
            if p.playerid not in [self.playerid, -2 - self.playerid, -1]:
              path = self.goto_poly(self.path[-1])
 
          if curpoly != None and curpoly != self.last_poly:
            #if curpoly.playerid not in [self.playerid, -1, -2 - self.playerid]:
            #  self.goto_poly(self.path[-1])
            #else:
              if self.last_poly.playerid == -2 - self.playerid:
                self.last_poly.playerid = -1
              self.last_poly.playeron = False
              self.last_poly = curpoly
              if self.last_poly.playerid == -1:
                self.last_poly.playerid = -2 - self.playerid
              self.last_poly.playeron = True
          #elif self.path_position < len(self.path) - 1 and \
          #     self.path[self.path_position+1].playerid not in [-1, self.playerid]:
          #  self.goto_poly(self.path[-1])
          if d < self.speed * 0.075: # we are close to the gpoint
            self.path_position = self.path_position + 1
            if curpoly.chargeball != None:
              curpoly.remove(curpoly.chargeball)
              if self.chargeball_val < 9:
                self.chargeball_val += curpoly.chargeball_val
                if self.chargeball_val > 9:
                  self.chargeball_val = 9
              curpoly.chargeball = None
              curpoly.chargeball_val = 0
            if self.path_position < len(self.path):    # do we have more checkpoints ?
              self.position = curpolypos
              self.sync_char()
              newpos = self.path[self.path_position].gpoint
              self.char.look_at(soya.Point(self.board, newpos[0], 0.0, newpos[1]))
            else:                                      # the journey is over
              self.status = 'idle'
              self.position = curpolypos
              self.sync_char()
              return
          self.position = self.get_new_pos(self.path[self.path_position].gpoint)
          self.sync_char()

        elif self.status == 'throwing':
          if curpoly == None:
            self.status = 'suspended'
            if self.chargeball_val == 0:
              self.game_over = True
              self.board.check_game_over()
            print 'Suspended'
          elif curpoly.playerid in [self.playerid, -1]:
            self.goto_poly(curpoly)
          else:
            self.last_poly = curpoly
            self.position = self.get_new_throw_pos()
            self.sync_char()

        elif self.status == 'suspended' and self.game_over:
          self.fallspeed += 0.01
          self.char.y -= self.fallspeed
          
    def set_last_poly(self):
        p = soya.Point(self.board)
        p.x=self.char.x
        p.y=0.1
        p.z=self.char.z
        v = soya.Vector(self.char, 0.0, -1.0, 0.0)
        self.last_poly = \
          self.board.raypick_polys(p, v)[0]
        self.last_poly.playeron = True

    def goto_poly(self, poly):
        if self.status == 'suspended':
          return []
        self.char.solid = 0
        p = soya.Point(self.board)
        p.x=self.char.x
        p.y=0.1
        p.z=self.char.z
        v = soya.Vector(self.char, 0.0, -1.0, 0.0)
        curpoly =  self.board.raypick_polys(p, v)[0]

        #print "gotopoly", poly.id, curpoly.id, poly.playeron
 
        path = self.board.path(curpoly, poly, self.playerid)
        if path == []:
            self.set_path([self.last_poly])
            #self.status = 'idle'
            return [self.last_poly]
        elif len(path)==1:
            if path[0].playerid in [-1, -2 - self.playerid, self.playerid]:
                self.set_path(path)
            else:
                self.set_path([self.last_poly])
                return [self.last_poly]
            return path
        if self.last_poly == path[1]:
            self.set_path(path[1:])
            return path[1:]
        self.set_path(path)
        return path

    def set_path(self, path):
        self.path = path
        self.path_position = 0
        self.status = 'moving'

    def board_changed(self):
        pass

    def throw(self, src):
        #TODO implement throw !!!
        if src.playerid == self.playerid:
          return
        self.throw_src = src
        self.char.look_at(soya.Point(self.board, src.gpoint[0], 0.0, src.gpoint[1]))
        self.last_poly.playeron = False
        self.status = 'throwing'


class AIPlayer(Player):
    def __init__(self, char, state):
        Player.__init__(self, char, state)

        # last visited polygon
        self.last_poly = None
        
        # the position on the path
        self.path_position = 0

        self.path = []
        self.char.begin_round = self.begin_round

    def begin_round(self):
        #curpoly = self.board.picking_context.raypick(self.char,
        #                                     self.state.camera.vector_to(self.char))
        self.power += self.powerinc
        if self.power > 100:
          self.power = 100
        p = soya.Point(self.board);p.x=self.char.x;p.y=0.1;p.z=self.char.z
        v = soya.Vector(self.char, 0.0, -1.0, 0.0)
        picres = self.board.raypick_polys(p, v)
        if picres:
          curpoly = picres[0]
          curpolypos = curpoly.gpoint
        else:
          curpoly = None
          curpolypos = (0.0, 0.0)

        if self.status == 'moving':
          d = dist(self.position, curpolypos)
          for p in self.path[self.path_position:]:
            if p.playerid not in [self.playerid, -2 - self.playerid, -1]:
              path = self.goto_poly(self.path[-1])
 
          if curpoly != None and curpoly != self.last_poly:
            #if curpoly.playerid not in [self.playerid, -1, -2 - self.playerid]:
            #  self.goto_poly(self.path[-1])
            #else:
              if self.last_poly.playerid == -2 - self.playerid:
                self.last_poly.playerid = -1
              self.last_poly.playeron = False
              self.last_poly = curpoly
              if self.last_poly.playerid == -1:
                self.last_poly.playerid = -2 - self.playerid
              self.last_poly.playeron = True
          #elif self.path_position < len(self.path) - 1 and \
          #     self.path[self.path_position+1].playerid not in [-1, self.playerid]:
          #  self.goto_poly(self.path[-1])
          if d < self.speed * 0.075: # we are close to the gpoint
            self.path_position = self.path_position + 1
            if curpoly.chargeball != None:
              curpoly.remove(curpoly.chargeball)
              if self.chargeball_val < 9:
                self.chargeball_val += curpoly.chargeball_val
                if self.chargeball_val > 9:
                  self.chargeball_val = 9
              curpoly.chargeball = None
              curpoly.chargeball_val = 0
            if self.path_position < len(self.path):    # do we have more checkpoints ?
              self.position = curpolypos
              self.sync_char()
              newpos = self.path[self.path_position].gpoint
              self.char.look_at(soya.Point(self.board, newpos[0], 0.0, newpos[1]))
            else:                                      # the journey is over
              self.status = 'idle'
              self.position = curpolypos
              self.sync_char()
              return
          self.position = self.get_new_pos(self.path[self.path_position].gpoint)
          self.sync_char()

        elif self.status == 'throwing':
          if curpoly == None:
            self.status = 'suspended'
            if self.chargeball_val == 0:
              self.game_over = True
              self.board.check_game_over()
            print 'Suspended'
          elif curpoly.playerid in [self.playerid, -1]:
            self.goto_poly(curpoly)
          else:
            self.last_poly = curpoly
            self.position = self.get_new_throw_pos()
            self.sync_char()

        elif self.status == 'suspended' and self.chargeball_val > 0:
          self.last_poly.playerid = self.playerid
          ChargeBall(self.board, self.last_poly, self.last_poly, self.chargeball_val)
          self.chargeball_val = 0

        elif self.status == 'idle':
          charging = False
          if curpoly.playerid == -2 - self.playerid:
            if self.power > 74:
              self.power -= 75
              curpoly.set_charge(self.playerid)
            else:
              charging = True
          
          if not charging:
            for n, ndist in curpoly.neighbours:
              if n.playerid == -1:
                self.goto_poly(n)

          mustcharge = False
          if self.status == 'idle' and not charging:
            for n, ndist in curpoly.neighbours:
              if n.playerid != self.playerid:
                mustcharge = True
                break
            if mustcharge and self.power > 74:
              self.power -= 75
              curpoly.set_charge(self.playerid)
              mustcharge = False
                
          if not charging and not mustcharge and self.status == 'idle':
            for polygon in self.board.polygons:
              if polygon.playerid == -1:
                path = self.goto_poly(polygon)
                self.status = 'idle'
                if path not in [[], [curpoly]]:
                  self.status = 'moving'
                  break

          if self.status == 'idle' and not mustcharge and not charging:
            for polygon in self.board.polygons:
              if polygon.playerid not in [-1, self.playerid]:
                for n, ndist in polygon.neighbours:
                  if n.playerid == self.playerid:
                    path1 = self.goto_poly(n)
                    if path1 != [curpoly]:
                      return

        elif self.status == 'suspended' and self.game_over:
          self.fallspeed += 0.01
          self.char.y -= self.fallspeed
                
            
    def set_last_poly(self):
        p = soya.Point(self.board)
        p.x=self.char.x
        p.y=0.1
        p.z=self.char.z
        v = soya.Vector(self.char, 0.0, -1.0, 0.0)
        self.last_poly = \
          self.board.raypick_polys(p, v)[0]
        self.last_poly.playeron = True

    def goto_poly(self, poly):
        if self.status == 'suspended':
          return []
        self.char.solid = 0
        p = soya.Point(self.board)
        p.x=self.char.x
        p.y=0.1
        p.z=self.char.z
        v = soya.Vector(self.char, 0.0, -1.0, 0.0)
        curpoly =  self.board.raypick_polys(p, v)[0]

        path = self.board.path(curpoly, poly, self.playerid)
        if path == []:
            self.set_path([self.last_poly])
            #self.status = 'idle'
            return [self.last_poly]
        elif len(path)==1:
            if path[0].playerid in [-1, -2 - self.playerid, self.playerid]:
                self.set_path(path)
            else:
                self.set_path([self.last_poly])
                return [self.last_poly]
            return path
        if self.last_poly == path[1]:
            self.set_path(path[1:])
            return path[1:]
        self.set_path(path)
        return path

    def set_path(self, path):
        self.path = path
        self.path_position = 0
        self.status = 'moving'

    def board_changed(self):
        pass

    def throw(self, src):
        #TODO implement throw !!!
        self.throw_src = src
        self.last_poly.playeron = False
        self.char.look_at(soya.Point(self.board, src.gpoint[0], 0.0, src.gpoint[1]))
        self.status = 'throwing'

