import soya
import soya.cube

from utils import dist

class ChargeBall(soya.World):
    def __init__(self, parent, srcpoly, destpoly, value):
        soya.World.__init__(self, parent)
        self.srcpoly = srcpoly
        self.destpoly = destpoly
        self.value = value
        self.board = self.parent
        
        #p = soya.Point(parent)
        #p.move(soya.Point(destpoly, destpoly.gpoint[0], 0, destpoly.gpoint[1]))
        #self.hpos = (p.x, p.z)

        plid = self.srcpoly.playerid
        
        if plid < 0:
            plid = -2 -plid
        
        self.playerid = plid
        self.distance = dist(srcpoly.gpoint, destpoly.gpoint)
        self.g = -10.0    # gravitational constant
        self.hspeed = 4.0 # horizontal speed is constant
        char_height = 1.0  # TODO we need the correct value for this
        
        d = self.distance
        
        if d == 0:
          if not destpoly.playeron: # we are on the edge
            self.destpoly.set_charge(self.playerid, inc=self.value)
            self.board.players[plid].status = 'moving'
            self.board.players[plid].last_poly = destpoly
            self.board.players[plid].set_path([destpoly])
          
          elif destpoly.playerid in [-1, self.playerid, -2 - self.playerid]:
            self.destpoly.set_charge(self.playerid, inc=self.value)
        
          self.parent.remove(self)
          return
        
        self.vspeed = -self.hspeed*(char_height+0.15+self.g*d**2/(2*self.hspeed**2))/d
        self.cvspeed = self.vspeed
        
        #self.set_xyz(srcpoly.gpoint[0], char_height, srcpoly.gpoint[1])
        
        self.move(soya.Point(srcpoly, srcpoly.gpoint[0], char_height+0.15, srcpoly.gpoint[1]))
        
        self.chargeball = soya.CylinderSprite(self, material = soya.Material.get('ball'))
        self.chargeball.lit = 0
        self.chargeball.y += 0.15
        self.chargeball.width = 0.20
        self.chargeball.height = 0.20
        self.chargeball.width += (value-1)*0.03
        self.chargeball.height += (value-1)*0.03
        self.chargeball.y += (value-1)*0.04

    def advance_time(self, proportion):
        self.y += self.cvspeed * 0.030 * proportion + self.g*(0.030*proportion)**2/2
        pass

    def begin_round(self):
        self.cvspeed = self.cvspeed + self.g * 0.030
        if dist((self.x, self.z), (self.destpoly.gpoint[0], self.destpoly.gpoint[1])) < 0.13:
          self.parent.remove(self)
          # here we charge the destination polygon
          if self.destpoly.playeron:
            plid = self.destpoly.playerid
            if plid < 0:
              plid = -2 -plid
            player = self.board.players[plid]
            if player.chargeball_val < 9:
              player.chargeball_val += self.value
              if player.chargeball_val > 9:
                player.chargeball_val = 9
          else:
            self.destpoly.set_charge(self.playerid, inc=self.value)
          return
        npos = self.get_new_pos(self.destpoly.gpoint)
        self.x = npos[0]
        self.z = npos[1]
        pass
        
    def get_new_pos(self, dest):
        x = self.x
        z = self.z
        p1 = soya.Point()
        p2 = soya.Point()
        p1.x = x
        p1.z = z
        p2.x = dest[0]
        p2.z = dest[1]
        v = p1.vector_to(p2)
        v.normalize()
        v = v * (self.hspeed * 0.03)
        x = x + v.x
        z = z + v.z
        return (x, z)
    

