#
# Overflow :: Network Classes
# by Team Soya (see AUTHORS)
#
# Copyright (C) 2005 Arc Riley
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#

import socket
from itertools import izip, repeat

class GenNet :
  def __init__(self, parent=None) :
    self.vers = 'Overflow-0.1'
    self.reset()
    self.z_index = 0
    self.parent = parent
    self.children = []
    if self.parent :
      self.parent.add_child(self)

  def add_child(self, child) :
    self.children.append(child)

  def begin_round(self) :
    for child in self.children :
      child.begin_round()

  def advance_time(self, prop) :
    for child in self.children :
      child.advance_time(prop)

  def end_round(self) :
    for child in self.children :
      child.end_round()

  def render(self) :
    for child in self.children :
      child.render()

  def on_resize(self) :
    for child in self.children :
      child.on_resize()

  def process_event(self, event) :
    for child in self.children :
      child.process_event(event)

  def error(self, errr) :
    print errr
    self.reset()
    self.errr = errr

  def reset(self) :
    self.errr = None
    self.sock = None			
    self.open = False			# True = ready to send cmds

  def connect(self, proto) :
    self.errr = None			# Reset error on (re)connect
    try :
      self.sock = socket.socket(socket.AF_INET, proto)
    except socket.error :
      self.reset()
      self.error('Couldn\'t create connection socket')
      return False
    return True

  def procline(self, commands) :
    line = self.recv()
    if not line :
      return True
    command = line[0][:2]
    if not commands.has_key(command) :
      return False
    args = self.getargs(line[0])
    if self.checkargs(command, args) :
      commands[command](args)
      return True
    return False

  def getargs(self, message):
    args = {}
    # Start by extracting the quoted argument, if present
    quotes = message.find('""')
    if quotes != -1:
      args['""'] = message[quotes+2:]
      message = message[:quotes-1]
    message = message.split()
    for word in message:
      args[word[:2].upper()] = word[2:]
    return args

  def checkargs(self, command, args):
    for req in self.required_args[command]:
      if not args.has_key(req):
        return False
    for arg in args:
      if self.format_args.has_key(arg[:2]) :
        vals = self.format_args[arg]
        if not vals[0](arg, args[arg], vals[1], vals[2]) :
          return False
    return True

  #
  # These are format checks for different arguments.  They are included
  # here since there's some overlap between the Zone and Game arguments
  # and it's much easier to have all the type check functs in one place.
  #
  def isnone(self, arg, val, minimum, maximum) :
    if len(val)>maximum :
      self.error('Value of %s exceeds maximum length (%s)' % (arg, maximum))
      return False
    if len(val)<minimum :
      self.error('Value of %s is under minimum length (%s)' % (arg, minimum))
      return False
    return True

  def isname(self, arg, val, minimum, maximum) :
    if not self.isnone(arg, val, minimum, maximum) :
      return False
    if not val[0].isalpha() :
      self.error('Value of %s begins with a non-letter' % (arg))
      return False
    if not val.replace('-','').replace('_','').replace('.','').isalnum() :
      self.error('Value of %s contains illegal character(s)' % (arg))
      return False
    return True

  def isdigi(self, arg, val, minimum, maximum) :
    if not val.isdigit() :
      self.error('Value of %s is not a number.' % (arg))
      return False
    if int(val)>maximum :
      self.error('Value of %s exceeds maximum allowed (%s)' % (arg, maximum))
      return False
    if len(val)<minimum :
      self.error('Value of %s is less than minimum allowed (%s)' % (arg, minimum))
      return False
    return True

  def isip46(self, arg, val, minimum, maximum) :
    # This needs to be written
    return True

  def islist(self, arg, val, minimum, maximum) :
    # This still needs to be written
    return True

  def cleanstrn(self, message) :
    xlat = dict(izip(xrange(31+1), repeat(None)))
    return message.translate(xlat)

  def getlist(self, message, type) :
    items = message.split(',')
    list = []
    for item in items :
      try :
        list.append(type(item))
      except :
        return ()

