#
# Overflow :: Game Network Client Class (GameClient)
# by Team Soya (see AUTHORS)
#
# Copyright (C) 2005 Arc Riley
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#

from time import time

from network.game import *

class GameClient(GameNet) :
  def __init__(self, parent=None, sock=None, nick='') :
    GameNet.__init__(self, parent)
    self.sock = sock
    if not self.sock :
      GameNet.connect(self)
    self.nick = nick
    self.required_args = {
      'HI' : ['BY'],
      'BO' : ['PG', 'TM', 'MX', '""'],
      'PG' : ['[]'],
      'TM' : ['[]'] }
    self.format_args = {
      'HI' : (self.isname, 3, 16),
      'BY' : (self.isname, 3, 64),
      'BO' : (self.isname, 3, 16),
      'PG' : (self.isdigi, 1, 4096),
      'TM' : (self.isdigi, 1, 6),
      'MX' : (self.isdigi, 1, 6),
      '[]' : (self.islist, 1, 8),
      '""' : (self.isnone, 0, 1023) }

  def reset(self) :
    print 'unbelievable ------------- a reset!'
    self.buff     = {}
    self.state    = 0
    self.board    = None
    self.server   = None
    self.wait     = 0
    self.resent   = 0
    self.teams    = []
    self.players  = {}
    self.misspoly = []
    self.missteam = []
    self.servername = ''


  def begin_round(self) :
    if self.state == 0 :
      #if not self.sock :
      #  print 'connecting listening socket'
      #  GameNet.connect(self)
      #else :
      self.state = 1
    if self.state == 1 and self.server :
      if not self.buff :
        self.buff[(self.server)] = ['', '']
      self.procline({'HI':self.recvHI})
    if self.state == 2 :
      self.procline({'HI':self.recvHI, 'BO':self.recvBO, 
                     'PG':self.recvPG, 'TM':self.recvTM})
      if not self.board :
        self.sendBO()
      else :
        if time()-self.wait > 3.0 :
          self.scanPG()
          self.scanTM()
        if self.misspoly :
          self.sendPG(self.misspoly)
        if self.missteam :
          self.sendTM(self.missteam)
        self.stage2check()
    if self.state == 3 :
      print 'state 3'
      self.procline({'PI':self.recvPI, 'PO':self.recvPO, 
                     'LG':self.recvLG, 'JO':self.recvJO,
                     'TK':self.recvTK, 'GS':self.recvGS })
    if self.state == 4 :
      pass      

  def recv(self) :
    line = GameNet.recv(self)
    if line :
      (line, address) = line
      if line[:2] == 'PI' and len(line.split()) == 3 :
        pong = line.split()[0][2:]
        dash = pong.find(':')
        if self.parent.nicks.has_key(pong[:dash]) :
          nick = self.parent.nicks[pong[:dash]]
          if str(nick.ping) == pong[dash+1:] :
            lag = str(time() - nick.ping)[:4]
            if nick.lag != lag :
              nick.lag = lag
              self.parent.serverlist()
          nick.myport = line.split()[2]
          if nick.myport.isdigit() :
            nick.myport = int(nick.myport)
          else :
            nick.myport = self.parent.port
      else :
        return (line, address)  
  #
  # STAGE 1
  #
  def recvHI(self, args) :
    print 'recv HI'
    if args['BY'] == self.vers :
      self.sendLO()
      self.state = 2
      self.wait = time()
    else :
      self.error('Server connected, but it\'s version didn\'t match')

  def sendLO(self) :
    self.send('LO%s BY%s' % (self.servername, self.vers), self.server)
    print 'sent LO'

  #
  # STAGE 2
  #
  def recvBO(self, args) :
    print 'recieved BO'
    if self.board :
      return
    self.board = board.Board(self.root_widget)
    self.board.title    = args['BO']
    self.board.polygons = [None] * int(args['PG'])
    self.board.teams    = int(args['TM'])
    self.board.members  = int(args['MX'])
    self.board.comment  = args['""']
    self.board.starts   = [None] * int(args['TM']) 
  
  def sendBO(self) :
    t = time()
    if t-self.wait > 1.0 and t-self.resent > 1.0 :
      self.send('BO', self.server)
      self.resent = t

  def recvPG(self, args) :
    if not self.board :
      self.sendBO(self)
      return
    id = int(args['PG'])
    if id < 1 or id > len(self.board.polygons) :
      return
    points = tuple(self.getlist(args['[]'], float))
    pointlist = []
    for i in xrange(len(points)/2):
      pointlist.append(points[i*2:i*2+2])
      print len(pointlist)
    self.board.polygons[id-1] = Poly(self.board, id-1, pointlist)
    if id > 1 and not self.board.polygons[id-2] :
      self.scanPG()
      

  def sendPG(self, ids) :
    if time()-self.resent > .25 :
      string = 'PG%d' % ids[0]
      for id in ids[1:] :
        string += '\nPG%d' % id
      self.send(string)
      self.resent = time()

  def scanPG(self) :
    self.misspoly = []
    if not self.misspoly :
      for id in xrange(len(self.teams)) :
        if not self.board.polygons[id] :
          self.misspoly.append(id+1)

  def recvTM(self, args) :
    if not self.board :
      self.sendBO(self)
      return
    id = int(args['TM'])
    if id < 1 or id > len(self.board.teams) :
      return
    points = tuple(self.getlist(args['[]'], int))
    if len(points) != self.board.players :
      return
    self.board.starts[id-1] = points
    if id > 1 and not self.board.starts[id-2] :
      self.scanTM()

  def sendTM(self, ids) :
    if time()-self.resent > .25 :
      string = 'TM%d' % ids[0]
      for id in ids[1:] :
        string += '\nTM%d' % id
      self.send(string)
      self.resent = time()

  def scanTM(self) :
    self.missteam = []
    if not self.missteam :
      for id in xrange(len(self.board.starts)) :
        if not self.board.starts[id] :
          self.missteam.append(id+1)

  def stage2check(self) :
    if self.board.polygons[-1] and self.board.starts[-1] :
      # If we got the last of both
      if not (self.misspoly or self.missteam) :
        # And we're not missing anything
        self.send('PG0\nTM0',self.server)
        self.state = 3

  #
  # STAGE 3
  #
  def recvPI(self, args) :
    self.sendPO(args['PI'])

  def sendPO(self, string) :
    self.send('PO%s' % string, self.server)
  
  def recvLG(self, args) :
    self.lag = args['LG']

  def recvJO(self, args) :
    team = args['JO']
    nick = args['FR']
    # Do something useful with this

  def sendJO(self, team) :
    self.send('JO%d' % team, self.server)

  def recvTK(self, args) :
    nick = args['FR']
    mesg = args['""']
    # Do something useful with this

  def sendTK(self, team) :
    self.send('TK ""%s' % message, self.server)

  def recvGS(self, args) :
    countdown = args['GS']
    # Do something useful with this
    self.sendGS()

  def sendGS(self) :
    self.send('GS', self.server)

