#
# Overflow :: Game Network Server Class
# by Team Soya (see AUTHORS)
#
# Copyright (C) 2005 Arc Riley
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#

from time import time

from network.game import *

class GameServer(GameNet) :
  def __init__(self, parent, board, client, desc) :
    GameNet.__init__(self, parent)
    self.board = board

    self.nick = client.nick
    self.sock = client.sock
    self.port = client.port
    # anything else we need to copy?
    self.desc = desc

    self.players = {}
    #self.connect()

  def begin_round(self) :
    if not self.sock :
      print '**************************************' , self.sock
      GameNet.connect(self)
    else :
      self.procline()

  def reset(self) :
    # Never 'reset' socket
    pass    

  def procline(self) :
    line = self.recv()
    if not line :
      return True
    address = line[1]
    line = line[0]
    print 'received %s from %s' % (line, address)
    # Pings are handled here.  All else goes to the gameplayer's buffer.
    if line[:2] == 'PI' :
      self.send('PO%s %s %s' % (line[2:], address[0], address[1]), address)
      print 'pinged, sent pong'
    if self.players.has_key(address) :
      self.players[address].buff[1].append('%s' % line)
    else :
      print line
    for player in self.players :
      self.players[player].begin_round()
      while self.players[player].buff[0] :
        print 'sending for', player
        self.send(self.players[player].buff[0].pop(), player)

  def addplayer(self, address, nick) :
    print 'ADDING %s at %s ' % (nick, address)
    self.players[address] = GamePlayer(self, nick)

  def remplayer(self, address) :
    if self.players.has_key(address) :
      self.players.pop(address)
       
class GamePlayer(GameNet) :
  def __init__(self, parent=None, nick='') :
    GameNet.__init__(self, parent)
    self.buff = [[], []] 			# [Send, Recv], both lists
    self.nick = nick
    self.state = 1
    self.resent = 0
    self.required_args = {
      'LO' : ['BY'],
      'BO' : [],
      'PG' : [],
      'TM' : [] }
    self.format_args = {
      'LO' : (self.isname, 3, 16),
      'BY' : (self.isname, 3, 64),
      'BO' : (self.isnone, 0, 0),
      'PG' : (self.isdigi, 0, 4096),
      'TM' : (self.isdigi, 0, 6),
      'PO' : (self.isnone, 1, 64),
      'JO' : (self.isdigi, 1, 6),
      'TK' : (self.isnone, 0, 0),
      '""' : (self.isnone, 0, 1023) }

  def reset(self) :
    # Never 'reset' socket
    pass    

  def send(self, message) :
    print 'send: ', self.buff[0]
    print '> %s\n' % message
    self.buff[0].append(message)

  def recv(self) :
    if self.buff[1] :
      print 'recv: ', self.buff[1]
      return '%s\n' % self.buff[1].pop(0)
    return None

  def begin_round(self) :
    if self.state == 1 :
      self.sendHI()
      self.procline({'LO':self.recvLO})      
    if self.state == 2 :
      self.procline({'BO':self.recvBO, 'PG':self.recvPG, 'TM':self.recvTM})
      self.stage2check()
    if self.state == 3 :
      self.procline({'PO':self.recvPO, 'JO':self.recvJO, 'TK':self.recvTK})
      self.stage3check()

  def checkveri(self) :
    for v in self.veri :
      if not v :
        return False
    return True


  #
  # STAGE 1
  #
  def sendHI(self) :
    if time() - self.resent < 3.0 :
      return
    self.send('HI%s BY%s' % (self.nick, self.vers))
    self.resent = time()
    print 'sent HI'  

  def recvLO(self, args) :
    if args['LO'] != self.parent.nick :
      print 'Client responded, but believes you\'re someone else (%s)' \
            % args['LO']
    if args['BY'] != self.vers :
      print 'Client responded, but it\'s version didn\'t match (%s)' \
            % args['BY']
    self.state = 2
    print 'state = 2'
    self.wait = time()
    # self.veri = polygons, teams
    self.veri = [False] * 2   
    self.sendBO()


  #
  # STAGE 2
  #
  def sendBO(self) :
    print 'sent BO!!!'  
    self.send('BO%s PG%d TM%d MX%d ""%s' % (self.parent.board.title, 
                                            len(self.parent.board.polygons), 
                                            self.parent.board.teams, 
                                            self.parent.board.members))

  def recvBO(self) :
    print 'recv BO!!!'  
    if time() - self.resent < 3.0 :
      print 'resent too quickly, waiting'
      return
    self.sendBO()
    self.resent = time()

  def selfPG(self, args) :
    string = ''
    for id in args['PG'] :
      pointlist = self.parent.board.polygons[id-1].points
      points = ''
      for point in pointlist :
        points += '%d,%d,' % (point[0], point[1])
      string += 'PG%d []%s\n' % (id, points[:-1])
    self.send(string[:-1])

  def recvPG(self, id) :
    if id == 0 :
      self.veri[0] = True
    else :
      self.sendPG((id,))

  def sendTM(self, args) :
    string = ''
    for id in args['TM'] :
      starts = str(self.parent.board.starts[id-1]).replace(' ','')[1:-1]
      string += 'TM%d []%s\n' % (id, starts)
    self.send(string[:-1])

  def recvTM(self, id) :
    if id == 0 :
      self.very[1] = True
    else :
      self.sendTM((id,))

  def stage2check(self) :
    if not checkveri() :
      return
    self.state == 3
    print 'state = 3'
    self.wait = time()
    # self.veri = polygons, teams
    self.veri = [False] * 2   
    self.ping = 0.0
    self.joined = []
    self.sendPI()


  #
  # STAGE 3
  #
  def sendPI(self) :
    self.ping = time()
    self.send('PI%f' % self.ping)

  def recvPO(self, args) :
    if args['PO'] != self.ping :
      return
    self.lag = time - self.ping
    self.sendLG()      

  def sendLG(self) :
    string = 'LG%f\n' % self.lag
    if stage == 3 :
      # resend JO's
      pass
    if stage == 4 :
      # resend PT's
      pass
    self.send(string[:-1])

# Ok now we need multiuser stuff... the stuff up top needs to be finished..
# or we're all stuffed...  yea, it's getting late.

  def recvJO(self, args) :
    if args['JO'] > self.parent.board.teams :
      return
    # !!!!! Need to make sure the team isn't full already !!!!!
    self.team = JO
    for p in self.parent.players :
      self.parent.players[p].sendJO(self)

  def sendJO(self, player) :
    self.send('JO%d FR%s' % (player.team, player.name))

  def recvTK(self, args) :
    for p in self.parent.players :
      self.parent.players[p].sendTK(self.nick, self.cleanstrn(args['""']))

  def sendTK(self, nick, message) :
    self.send('TK FR%s ""%s' % (nick, message))

  def stage3check(self) :
    print 'state = 4'
    pass

  #
  # STAGE 4
  #

