#
# Overflow :: Metaserver Network Class (ZoneNet)
# by Team Soya (see AUTHORS)
#
# Copyright (C) 2005 Arc Riley
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#

import widgets
from time import time, ctime, tzname, daylight

from network import *
from network.client import *
from network.server import *

Client = 1
Server = 2

class ZoneNet(GenNet) :
  def __init__(self, parent, state, nick='', key='') :
    GenNet.__init__(self, parent)
    self.state = state
    self.nick = nick
    self.key  = key
    self.buff = ['','']		# [Send Buffer, Recv Buffer]
    self.wins = {'main':ChatWindow(), 'room':ChatWindow()}
    self.srvs = [''] * 4
    self.meta = 'meta.gnuzone.org'
    self.gamelabel = 'OVERFLW0'
    self.mode = Client
    self.game = GameClient()
    self.required_args = {	# These are arguments which MUST be sent
      'LO' : ['IM', 'BY'],	# as part of different commands.  If one
      'GO' : [],		# or more are missing, the server is not
      'GA' : ['""'],		# working properly and the client should
      'ON' : ['IP'],		# disconnect from it.  
      'OF' : [],			
      'JO' : ['FR'],
      'PT' : ['FR'],
      'TL' : ['FR', '""'],
      'TK' : ['FR', '""'],
      'YL' : ['FR', '""'],
      'GL' : ['FR', '""'],
      'PI' : [],
      'NO' : [] }
    self.format_args = {
      'LO' : (self.isname, 3, 16),
      'IM' : (self.isname, 1, 255),
      'BY' : (self.isname, 3, 64),
      'GO' : (self.isname, 1, 255),
      'PO' : (self.isdigi, 1, 65535),
      'GA' : (self.isname, 3, 8),
      'MX' : (self.isdigi, 0, 999),
      'ON' : (self.isname, 3, 16),
      'CU' : (self.isdigi, 0, 999),
      'IP' : (self.isip46, 0, 255),
      'OF' : (self.isname, 3, 16),
      'JO' : (self.isname, 3, 8),
      'FR' : (self.isname, 3, 16),
      'PT' : (self.isname, 3, 8),
      'TL' : (self.isname, 3, 16),
      'TK' : (self.isname, 3, 8),
      'YL' : (self.isnone, 0, 0),
      'GL' : (self.isnone, 0, 0),
      'PI' : (self.isnone, 0, 65536),
      'NO' : (self.isnone, 0, 1023),
      '""' : (self.isnone, 0, 1023) }
    self.queries = {
      'PING' : self.queryPING,
      'PLAY' : self.queryPLAY,
      'TIME' : self.queryTIME,
      'VERS' : self.queryVERS }
    if nick and key :
      self.connect()
      
  def send(self, message) :
    if not self.sock :
      self.reset()
      return False
    message = '%s%s\n' % (self.buff[0], message)
    try :
      sent = self.sock.send(message)
    except socket.error :
      self.reset()
      return False
    self.buff[0] = message[sent:]
    return True

  def recv(self) :
    if not self.sock :
      self.reset()
      return False
    try : 
      recv = self.sock.recv(1024)
    except socket.error :
      recv = ''
    self.buff[1] += recv
    newline = self.buff[1].find('\n')
    if newline != -1 :
      line = self.buff[1][:newline]
      self.buff[1] = self.buff[1][newline+1:]
      return (line, None)
    else :
      return None

  def reset(self):
    GenNet.reset(self)
    self.joind = False
    self.games = {}
    self.nicks = {}
    self.buff = ['','']		# [Send Buffer, Recv Buffer]

  def connect(self) :
    if not GenNet.connect(self, socket.SOCK_STREAM) :
      return False
    try :
      self.sock.connect(('72.43.24.246', 1010))
      self.sock.setblocking(0)
    except socket.error :
      self.reset()
      self.error('Couldn\'t connect to %s on port %s' % ('72.43.24.246', 1010))
      return False
    if not self.send('HI%s IM%s:%s BY%s'%(self.meta, 
                     self.nick, self.key, self.vers)) :
      self.error('Metaserver connection closed before greeting could be sent')

  def begin_round(self) :
    GenNet.begin_round(self)
    if not self.sock :
      return
    if not self.open :
      if not self.procline({'LO':self.recvLO, 'NO':self.recvNO }) : 
        self.reset()
        self.error('Unrecognised connect response from metaserver')
    else :
      self.procline({'GA':self.recvGA, 'ON':self.recvON, 
                     'OF':self.recvOF, 'JO':self.recvJO,
                     'PT':self.recvPT, 'TL':self.recvTL, 
                     'TK':self.recvTK, 'YL':self.recvYL, 
                     'GL':self.recvGL, 'PI':self.recvPI, 
                     'NO':self.recvNO })
      self.procgame()
    self.game.begin_round()

  def procgame(self) :
    if self.mode == Client :
      self.procclient()
    elif self.mode == Server : 
      self.procserver()

  def procclient(self) :
    if not self.game.sock :
      if self.nick :
        self.game.nick = self.nick
    
    # Send new pings
    for nick in self.nicks :
      if self.nicks[nick].game.upper() != self.gamelabel :
        continue
      if time() - self.nicks[nick].ping < 30.0 :
        continue
      self.nicks[nick].ping = time()
      self.game.send('PI%s:%s' % (nick, self.nicks[nick].ping), 
                     (self.nicks[nick].ip, self.nicks[nick].port))

  def procserver(self) :
    pass

  def chatline(self, window, line) :
    self.wins[window].chatbuff.append(self.cleanstrn(line))

  def userlist(self, window) :
    if window == 'main' :
      users = self.nicks
    else :
      users = self.games[self.gamelabel].chat
    list = []
    sorted = users.keys()
    sorted.sort()
    for user in sorted :
      list.append(users[user].name)
    self.wins[window].userlist.replace(list)

  def serverlist(self) :
    games = []
    nicks = self.nicks.keys()
    nicks.sort()
    for nick in nicks :
      if self.nicks[nick].game.upper() == self.gamelabel :
        cu = self.nicks[nick].cur
        if not cu :
          cu = '-'
        mx = self.nicks[nick].max
        if not mx :
          mx = '-'
        games.append((self.nicks[nick].name, self.nicks[nick].lag,
                      '%s/%s' % (cu, mx), self.nicks[nick].desc)) 
        if len(games) < 5 :
          games.extend((('') * 4) * (5 - len(games)))
        self.srvs.replace(games)

  def recvLO(self, args) :
    self.nick = args['LO']
    self.serv = '%s (%s)' % (args['IM'], args['BY'])
    self.open = True
    self.chatline('main', u'Welcome to GNUZone.')
    self.chatline('main', u'')

  def recvGA(self, args) :
    self.games[args['GA'].upper()] = ZoneGame(args['GA'], args['""'])
    if args.has_key('MX') :
      self.games[args['GA'].upper()].max = args['MX']
    if args['GA'].upper() == self.gamelabel :
      self.send('JO%s' % args['GA'])
    return True

  def recvON(self, args) :
    self.nicks[args['ON'].upper()] = ZoneNick(args['ON'], args['IP'])
    if args.has_key('GA') :
      self.nicks[args['ON'].upper()].game = args['GA']
      if args.has_key('PO') :
        self.nicks[args['ON'].upper()].port = int(args['PO'])
      else :
        self.nicks[args['ON'].upper()].port = 20294
      if args.has_key('CU') :
        self.nicks[args['ON'].upper()].cur = args['CU']
        if args.has_key('MX') :
          self.nicks[args['ON'].upper()].max = args['MX']
      if args.has_key('""') :
        self.nicks[args['ON'].upper()].desc = args['""']
      self.serverlist()
    return True

  def recvOF(self, args) :
    if self.nicks.has_key(args['OF'].upper()) :
      self.dooff(args['OF'])
    return True

  def recvJO(self, args) :
    if self.games.has_key(args['JO'].upper()) and \
       self.nicks.has_key(args['FR'].upper()) :
      self.dojoin(args['JO'], args['FR'])
    return True

  def recvPT(self, args) :
    if self.games.has_key(args['PT'].upper()) and \
       self.games[args['PT'].upper()].chat.has_key(args['FR'].upper()) :
      self.dopart(args['PT'], args['FR'])
    return True



  def recvTL(self, args) :
    if not self.querycheck(args['FR'], args['""']) :
      self.chatline('room', u'*%s* %s' % (args['FR'], args['""']))
    return True

  def recvTK(self, args) :
    if not self.querycheck(args['FR'], args['""']) :
      if args['TK'].upper() == self.gamelabel :
        self.chatline('room', u'<%s> %s' % (args['FR'], args['""']))
    return True

  def recvYL(self, args) :
    self.chatline('main', u'<%s> %s' % (args['FR'], args['""']))
    return True

  def recvGL(self, args) :
    self.chatline('main', u'[%s] %s' % (args['FR'], args['""']))
    return True

  def recvPI(self, args) :
    self.send('PO%s' % args['PI'])
    return True

  def recvNO(self, args) :
    if args['NO'] == 'HI' :
      self.reset()
      # should be a bit more graphically verbose
      self.error('Server refused greeting: "%s"' % args['""'])
      return False
    return True

  def dooff(self, nick) :
    self.nicks.pop(nick.upper()) and None
    for room in self.games :
      if self.games[room].chat.has_key(nick.upper()) :
        self.games[room].chat.pop(nick.upper()) and None
        if room.upper() == self.gamelabel :
          self.chatline('room', u'*** %s has logged off.' % nick)
          self.userlist('room')
    self.userlist('main')

  def dojoin(self, room, nick) :
    self.games[room.upper()].chat[nick.upper()] = self.nicks[nick.upper()]
    if room.upper() == self.gamelabel :
      if nick.upper() == self.nick.upper() :
        self.joind = True
      if self.joind :
        self.chatline('room', u'*** %s has joined the room.' % nick) 
        self.userlist('room') 
      if self.mode == Server :
        self.sendPI(nick)

  def sendPI(self, nick) :
    # Send an array of packets to open connection to 
    for p in range(20294, 20300) :
      self.game.send('PI%f' % time(), (self.nicks[nick.upper()].ip, p))

  def dopart(self, room, nick) :
    self.games[room.upper()].chat.pop(nick.upper()) and None
    if room.upper() == self.gamelabel :
      self.chatline('room', u'*** %s has left the room.' % nick) 
      self.userlist('room') 
      if nick == self.nick :
        self.joind = False

  def sendYL(self, message) :
    self.send('YL ""%s' % message)

  def sendTK(self, message) :
    if self.joind :
      self.send('TK%s ""%s' % (self.gamelabel, message))
    else :
      self.chatline('room', u'!!! Can\'t send, not joined to game room !!!')

  def sendTL(self, nick, message) :
    self.send('TL%s ""%s' % (nick, message))

  def connectserv(self, servername) :
    if self.nicks.has_key(servername.upper()) :
      self.sendTL(servername, '?PLAY %s' % self.game.port)
      self.game.server = (self.nicks[servername.upper()].ip,
                          self.nicks[servername.upper()].port)
      self.game.servername = servername

  def querycheck(self, fr, line) :
    if line[0] == '?' :
      query = line[1:].split()
      if not query :
        return False
      if self.queries.has_key(query[0]) :
        self.queries[query[0]](fr, query[1:])
      return True
    return False

  def queryPING(self, fr, query) :
    if query :
      self.send('TL%s "".PING %s' % (fr, query[0]))
    else :
      self.send('TL%s "".PING' % fr)

  def queryPLAY(self, fr, query) :
    if not self.mode == Server :
      self.send('TL%s "".PLAY I\'m not currently hosting a game.' % fr)
      return
    if query :
      if not query[0].isdigit() :
        self.send('TL%s "".PLAY I can\'t connect to a non-numeric port.' % fr)
        return
    port = int(query[0])
    self.send('TL%s "".PLAY Attempting to connect on port %d...' % (fr, port))
    self.game.addplayer((self.nicks[fr.upper()].ip, port), fr)


  def queryTIME(self, fr, query) :
    self.send('TL%s "".TIME %s %s' % (fr, ctime(time()), tzname[daylight]))

  def queryVERS(self, fr, query) :
    self.send('TL%s "".VERS %s' % (fr, self.vers))

  def startserv(self, board, desc) :
    print 'Changing to server mode'
    if self.mode != Client :
      print 'Not a client?'
      return
    self.game = GameServer(self, board, self.game, desc)
    self.mode = Server
    max = board.teams * board.members
    self.send('ON GA%s PO%d CU1 MX%s ""%s' % (self.gamelabel, 
              self.game.port, max, desc))
    for nick in self.nicks :
      self.sendPI(nick)



#
# Classes for ZoneNet
#
class ChatWindow :
  # Ok this is definetly a hack...
  class bufferclass :
    def append(self, item) :
      return
    def extend(self, items) :
      return
    def replace(self, items) :
      return
  def __init__(self) :
    self.userlist = self.bufferclass()
    self.chatbuff = []

class ZoneGame :
  def __init__(self, name, desc) :
    self.name = name
    self.desc = desc
    self.max  = 0
    self.chat = {}

class ZoneNick :
  def __init__(self, name, ip, port=20294) :
    self.name = name
    self.ip   = ip
    self.game = ''
    self.port = port
    self.cur  = 0
    self.max  = 0
    self.desc = ''
    self.lag  = '-'
    self.ping = 0
    self.myport = 0
