import PIL.Image

import soya
from soya import pudding
from soya.pudding.ext.slicingimage import SlicingImage
from soya.opengl import *

import states
import utils
import widgets

class BaseMenuState(states.ApplicationState):
  """ Base class for all menu states """

  class MenuContainer(pudding.container.VerticalContainer):
    """ Menu widget """

    def __init__(self, root_widget):
      pudding.container.VerticalContainer.__init__(self,
                                          root_widget,
                                          left = 50,
                                          top = 40,
                                          width = 200,
                                          height = 200,
                                          padding = 10)

      self.animating = True
      self.animating2 = False
      self.anim_width = 40.
      self.anim_height = 40.
      self.anim_alpha = 0.

    def render_self(self):
      if not self.animating:
        pudding.container.VerticalContainer.render_self(self)

        glTranslatef(-20, -10, 0)
        pudding.STYLE.draw_bordered_box(self.width + 20, self.height + 20) 

      else:
        glTranslatef(-20, -10, 0)
        pudding.STYLE.draw_bordered_box(self.anim_width + 20, self.anim_height + 20) 

    def advance_time(self, p):
      if self.animating:
        if int(self.anim_width) != self.width:
          self.anim_width += p * .2 * (self.width - self.anim_width)
        if int(self.anim_height) != self.height:
          self.anim_height += p * .2 * (self.height - self.anim_height)

        if int(round(self.anim_width)) == self.width and int(round(self.anim_height)) == self.height:
          self.animating = False
          self.animating2 = True
        return 

      elif self.animating2:
        self.anim_alpha += p * .1 

        if self.anim_alpha > 1.:
          self.animating2 = False
        
        color = (pudding.STYLE.BLUE1[0],
                  pudding.STYLE.BLUE1[1],
                  pudding.STYLE.BLUE1[2],
                  self.anim_alpha)

        for child in self.children:
          child.color = color
        return 

      else:
        pudding.container.VerticalContainer.advance_time(self, p)

    def process_event(self, event):
      if self.animating:
        return 

      pudding.container.VerticalContainer.process_event(self, event)


  class MenuOption(pudding.control.Label):
    """ Menu widget option """
    def __init__(self, label, function):
      pudding.control.Label.__init__(self, label = label)
      
      self.function = function
      
      self.color2 = (0.0, 0.0, 0.0, 0.9)
      self.color1 = pudding.STYLE.BLUE1
      self.color = (0.0, 0.0, 0.0, 0.0)

    def on_focus(self):
      self.color = self.color2

    def on_loose_focus(self):
      self.color = self.color1

    def on_mouse_up(self, button, x, y):
      if self.function:
        self.function()
      else:
        print "Unhandled Click! %s" % self.label

      return True
  
  def setup_scene(self):
    states.ApplicationState.setup_scene(self)

    #m_splash = PIL.Image.open(utils.find_image_in_path('menu_tmp.png'))
    #background = SlicingImage(self.root_widget, 
    #                            m_splash,
    #                            left = 0, top = 0,
    #                            )

    background = widgets.Background(self.root_widget)

    self.menu = self.MenuContainer(self.root_widget)
    self.setup_menu()

  def setup_menu(self):
    """ actuall add options to the menu widget using 
    self.get_menu_options """

    for child in self.get_menu_options():
      if type(child) is tuple:
        child, mode = child
      else:
        mode = pudding.ALIGN_LEFT

      self.menu.add_child(child, mode)

  def get_menu_options(self):
    """ override this - return MenuOption's """
    return []


