import PIL.Image

import soya
from soya import pudding 
from soya import sdlconst as sdl
from soya.pudding.ext.slicingimage import SlicingImage

import states
import utils
import widgets

from states.intro import IntroState, IntroImage, RootWidget

class RootWidget2(RootWidget):
  def on_mouse_up(self, x, y, button):
    self.state.do_image()
    self.state.clock = 0

  def on_key_self(self, key, mods):
    if key == sdl.K_ESCAPE:
      self.state.finish()
    elif key < 255:
      self.state.do_image()
      self.state.clock = 0
      

class HelpState(IntroState):
  """ State for the beggining of the game, before the main menu """

  ROOT_WIDGET_CLASS = RootWidget2

  def setup_scene(self):
    states.ApplicationState.setup_scene(self)

    self.root_widget.children.remove(self.logo)
    
    images = [ 
              'help1.png',
              'help2.png',
              'help3.png',
              'help4.png',
              'help5.png',
              'help6.png',
              'help7.png',
              'help8.png',
              'help9.png',
             ]


    self.images = []

    for image in images:
      splash_image = PIL.Image.open(utils.find_image_in_path(image))
      splash_image.convert('RGBA')

      splash = IntroImage(self.root_widget, 
                                splash_image,
                                left = 0, top = 0,
                                right = 0, bottom = 0,
                                )
      splash.anchors = pudding.ANCHOR_ALL

      self.images.append(splash)

    self.last_splash = None
    self.current_image = -1

    self.clock = 0.

    self.finishing = False

    self.do_image()

  def begin_round(self):
    states.ApplicationState.begin_round(self)
    self.clock += 1

    if not self.finishing and self.clock > 300:
      self.do_image()
      self.clock = 0 
    
