from random import random

import PIL.Image

import soya
from soya import sdlconst as sdl
from soya import pudding 
from soya.pudding.ext.slicingimage import SlicingImage
from soya.opengl import *

import board
import states
import utils
import network.zonenet
import widgets

from states.menu_local import GameList as BoardListWindow


class PreviewCamera(soya.Camera):
  def __init__(self, parent, left, top, width, height):
    soya.Camera.__init__(self, parent)
    self.set_viewport(left, top, width, height)
    self.partial = 1
  def resize(self, left, top, width, height):
    # Overwritting this method allows us to make this a small camera
    pass


class ChatTextWindow(pudding.control.Panel):
  class ChatTextBuffer(widgets.ScrollingBuffer):
    def __init__(self, parent = None, **kwargs):
      widgets.ScrollingBuffer.__init__(self, parent, 
                              style = widgets.ScrollingBuffer.FROM_BOTTOM, 
                              **kwargs)

      self.scroll_bar.inverted = True

  def __init__(self, parent, **kwargs):
    pudding.control.Panel.__init__(self, parent, **kwargs)

    self.text = self.ChatTextBuffer(self, left = 10, 
                                          top = 10, 
                                          right = 10,
                                          bottom = 10
                                          )

  def on_resize(self):
    pudding.control.Panel.on_resize(self)
    self.text.resize(0, 0, self.width, self.height)

class ServerChatWindow(ChatTextWindow):
  pass

class RoomChatWindow(ChatTextWindow):
  pass

class GameChatWindow(ChatTextWindow):
  pass


class UserListWindow(pudding.control.Panel):
  class UserList(widgets.ScrollingBuffer):
    pass

  def __init__(self, parent, **kwargs):
    pudding.control.Panel.__init__(self, parent, **kwargs)

    self.list = self.UserList(self, left = 10, 
                                        top = 10, 
                                        right = 10,
                                        bottom = 10,
                                        )

  def on_resize(self):
    pudding.control.Panel.on_resize(self)
    self.list.resize(0, 0, self.width, self.height)


class GameListWindow(UserListWindow):
  def __init__(self, parent, **kwargs):
    pudding.control.Panel.__init__(self, parent, **kwargs)

    self.list = self.UserList(self, left = 10, 
                                    top = 10, 
                                    right = 10, 
                                    bottom = 10, 
                                    buffer_class = \
                                     widgets.MultiColumnScrollBufferText)


class GamePreviewWindow(pudding.control.Box):
  pass

class TextInput(pudding.control.Input):
  def __init__(self, parent, state, **kwargs):
    pudding.control.Input.__init__(self, parent, **kwargs)

    self.label.font = pudding.STYLE.small_font
    self.label.color = pudding.STYLE.BLUE1

    self.state = state

    self.label.left += 10
    self.label.right += 10
    self.label.anchors = pudding.ANCHOR_ALL
    
    self.height = 50

    self.label.post = self._cursor
    
    self.label.update()

  def on_key_down(self, key, mods):
    if key > 256:
      return False

    return pudding.control.Input.on_key_down(self, key, mods)
  
  def on_return(self):
    if self.state.server_chat_window.visible :
      self.state.zone_net.sendYL(self.value)
    elif self.state.room_chat_window.visible :
      self.state.zone_net.sendTK(self.value)
    elif self.state.room_chat_window.visible :
      print 'cant send to game yet'
      return
    else :
      return    
    self.clear()

  def on_resize(self):
    pudding.control.Input.on_resize(self)
    self.label.top += 8

  def on_focus(self):
    pass

  def on_loose_focus(self):
    pass
    

class TabButton(pudding.control.Button):
  def __init__(self, parent = None, func = None, **kwargs):
    pudding.control.Button.__init__(self, parent, **kwargs)

    self.color1 = pudding.STYLE.BLUE2
    self.color2 = pudding.STYLE.BLUE1

    self.label_control.color = self.color1
    self.label_control.font = pudding.STYLE.small_font

    self.func = func

  def set_state(self, state):
    if state:
      self.label_control.color = self.color2
    else:
      self.label_control.color = self.color1

  def on_click(self):
    if self.func:
      self.func()

  def on_resize(self):
    pudding.control.Button.on_resize(self)
    self.label_control.top -= 13

  def render_self(self):
    pudding.STYLE.draw_bordered_box(self.width, self.height, tab = True)

class RootWidget(widgets.RootWidget):
  def on_key_down(self, key, mods):
    if key == sdl.K_ESCAPE:
      self.state.try_exit()
      return True

    else:
      return self.state.text_input.on_key_down(key, mods)

    return False

  def on_resize(self):
    widgets.RootWidget.on_resize(self)

    self.state.on_resize()


class KioskState(states.ApplicationState):
  """ main chatroom/kiosk state """

  ROOT_WIDGET_CLASS = RootWidget

  def __init__(self, application):
    self.widgets_ok = False

    states.ApplicationState.__init__(self, application)
    self.setup_widgets()

    self.on_resize()

  def setup_widgets(self):
    background = widgets.Background(self.root_widget)

    self.chat_buttons = pudding.container.HorizontalContainer(self.root_widget,
                                  left = 70, 
                                  top = 50,
                                  width = 300, 
                                  height = 60,
                                  z_index = 0
                                  )

    self.chat_buttons.add_child(TabButton(label = 'GNUZone\nEveryone', 
      func = self.activate_server), pudding.EXPAND_BOTH)
    self.chat_buttons.add_child(TabButton(label = 'GNUZone\nOverflow', 
      func = self.activate_room), pudding.EXPAND_BOTH)
    self.chat_buttons.add_child(TabButton(label = 'Game\nServer', 
      func = self.activate_game), pudding.EXPAND_BOTH)

    self.tab_but_server = self.chat_buttons.children[0]
    self.tab_but_room = self.chat_buttons.children[1]
    self.tab_but_game = self.chat_buttons.children[2]

    self.tab_but_server.set_state(True)

    self.server_chat_window = ServerChatWindow(self.root_widget, 
                                  left = 10,
                                  top = 10,
                                  width = 400,
                                  height = 400,
                                  )

    self.room_chat_window = RoomChatWindow(self.root_widget, 
                                  left = 10,
                                  top = 10,
                                  width = 400,
                                  height = 400,
                                  visible = False,
                                  )

    self.game_chat_window = RoomChatWindow(self.root_widget, 
                                  left = 10,
                                  top = 10,
                                  width = 400,
                                  height = 400,
                                  visible = False,
                                  )

    self.text_input = TextInput(self.root_widget, self,  
                                  left = 10,
                                  top = 410,
                                  width = 400,
                                  )

    self.user_window = UserListWindow(self.root_widget,
                                  left = 410,
                                  height = 400,
                                  top = 10,
                                  width = 300,
                                  )

    self.preview_window = GamePreviewWindow(self.root_widget, )

    self.game_window = GameListWindow(self.root_widget,)
    self.board_window = BoardListWindow(self.root_widget, )
    self.board_window.list.extend(board.get_board_files())
    self.board_window.visible = False
    self.my_server_window = UserListWindow(self.root_widget,)
    self.my_server_window.visible = False

    self.connect_but = widgets.Button(self.root_widget, 
                                                label = 'Connect',
                                                func = self.do_connect,
                                                left = 10, 
                                                top = 10, 
                                                height = 50,
                                                )

    self.host_but = widgets.Button(self.root_widget, 
                                                label = 'Host Game',
                                                func = self.do_host,
                                                left = 10, 
                                                top = 10, 
                                                height = 50,
                                                )

    self.widgets_ok = True

    if self.application.config.has_section('gnuzone'):
      nick = self.application.config.get('gnuzone', 'nick')
      key = self.application.config.get('gnuzone', 'key')

      #prompt = widgets.MeterPrompt(text = 'some status', width = 300, height = 200)
    else:
      prompt = widgets.LoginPrompt(text = 'Login to GNUZone',
                                          callback = self.done_login)
      nick = key = ''

    self.zone_net = network.zonenet.ZoneNet(self.root_widget, self, nick = nick, key = key)

    self.zone_net.wins['main'].chatbuff = self.server_chat_window.text
    self.zone_net.wins['room'].chatbuff = self.room_chat_window.text
    self.zone_net.wins['room'].userlist = self.user_window.list
    self.zone_net.srvs = self.game_window.list

  def do_connect(self):
    selection = self.game_window.list.text.selection

    if not selection:
      return 

    servername = selection.children[0].label.upper()
    print servername

    if not servername:
      return 

    self.zone_net.connectserv(servername)
   

  def do_host(self):
    self.game_window.visible = False
    self.board_window.visible = True

    self.host_but.label = 'Select'
    self.host_but.func = self.do_select_board
    self.connect_but.label = 'Cancel'
    self.connect_but.func = self.do_cancel_host

  def do_select_board(self):
    selection = self.board_window.list.text.selection

    if not selection:
      widgets.Prompt(text = 'Please select a board', width = 300, height = 200)
      return 

    board_name  = selection.label
    servboard = board.Board(self.preview_world, self, board_name)
    #
    # instead of sending servboard.comment, it *SHOULD* allow the user
    # to enter their own description for the game.  
    #
    self.zone_net.startserv(servboard, servboard.comment)

    self.board_window.visible = False
    self.host_but.visible = False
    self.connect_but.label = 'Start'
    self.connect_but.func = self.do_start_server
    self.my_server_window.visible = True

  def do_start_server(self):
    print "start server stuff!!!"

  def do_cancel_host(self):
    self.game_window.visible = True
    self.board_window.visible = False

    self.host_but.label = 'Host Game'
    self.host_but.func = self.do_host
    self.connect_but.label = 'Connect'
    self.connect_but.func = self.do_connect

   
  def done_login(self, prompt):
    if prompt.value:
      self.zone_net.nick = prompt.value[0]
      self.zone_net.key = prompt.value[1]

      if not self.application.config.has_section('gnuzone'):
        self.application.config.add_section('gnuzone')
        
      self.application.config.set('gnuzone', 'nick', self.zone_net.nick)
      self.application.config.set('gnuzone', 'key', self.zone_net.key)
      self.application.config.save()

      self.zone_net.connect()

      #prompt = widgets.MeterPrompt(text = 'some status', width = 300, height = 200)
    else:
      self.application.set_state(states.MainMenuState)

  def try_exit(self):
    prompt = widgets.YesNoPrompt(text = 'Do you really want to quit?', 
                              callback = self.exit_callback,
                              left = 10,
                              top = 10,
                              width = 400,
                              height = 150,
                              )

  def exit_callback(self, prompt):
    if prompt.value:
      self.application.set_state(states.MainMenuState) 

  def activate_server(self):
    self.server_chat_window.visible = True
    self.room_chat_window.visible = False
    self.game_chat_window.visible = False

    self.tab_but_server.set_state(True)
    self.tab_but_room.set_state(False)
    self.tab_but_game.set_state(False)

  def activate_room(self):
    self.server_chat_window.visible = False
    self.room_chat_window.visible = True
    self.game_chat_window.visible = False

    self.tab_but_server.set_state(False)
    self.tab_but_room.set_state(True)
    self.tab_but_game.set_state(False)
    
  def activate_game(self):
    self.server_chat_window.visible = False
    self.room_chat_window.visible = False
    self.game_chat_window.visible = True

    self.tab_but_server.set_state(False)
    self.tab_but_room.set_state(False)
    self.tab_but_game.set_state(True)
  
  def on_resize(self):
    if not self.widgets_ok:
      return 

    self.game_window.top = 10
    self.game_window.left = 10
    self.game_window.right = 300
    self.game_window.height = 180

    self.board_window.top = self.game_window.top
    self.board_window.left = self.game_window.left
    self.board_window.right = self.game_window.right
    self.board_window.height = self.game_window.height

    self.my_server_window.top = self.game_window.top
    self.my_server_window.left = self.game_window.left
    self.my_server_window.right = self.game_window.right
    self.my_server_window.height = self.game_window.height

    self.connect_but.left = 10
    self.connect_but.top = self.game_window.top + self.game_window.height + 5

    self.host_but.left = 150
    self.host_but.top = self.game_window.top + self.game_window.height + 5

    self.preview_window.top = 10
    self.preview_window.left = self.game_window.left + self.game_window.width + 10
    self.preview_window.right = 10
    self.preview_window.height = self.game_window.height

    self.preview_world = soya.World()
    self.preview_camera = PreviewCamera(self.preview_world,
                                        self.preview_window.left,
                                        self.preview_window.top,
                                        self.preview_window.right - \
                                        self.preview_window.left,
                                        self.preview_window.height )
    
    # this doesn't work.
    #self.root_widget.add_child(self.preview_camera)

    self.chat_buttons.left = 20
    self.chat_buttons.top = self.connect_but.top + self.connect_but.height + 5
    
    self.server_chat_window.top = self.chat_buttons.top + self.chat_buttons.height - 10
    self.server_chat_window.bottom = 100
    self.server_chat_window.right = self.game_window.right
    self.server_chat_window.left = 10

    self.room_chat_window.top = self.server_chat_window.top 
    self.room_chat_window.left = self.server_chat_window.left
    self.room_chat_window.width = self.server_chat_window.width
    self.room_chat_window.height = self.server_chat_window.height

    self.game_chat_window.top = self.server_chat_window.top
    self.game_chat_window.left = self.server_chat_window.left
    self.game_chat_window.width = self.server_chat_window.width
    self.game_chat_window.height = self.server_chat_window.height

    self.text_input.top = self.server_chat_window.top + self.server_chat_window.height + 10
    self.text_input.right = 240

    self.user_window.top = self.chat_buttons.top
    self.user_window.left = self.preview_window.left
    self.user_window.right = self.preview_window.right
    self.user_window.bottom = self.server_chat_window.bottom

    self.server_chat_window.text.text.reset_children = True
    self.room_chat_window.text.text.reset_children = True
    self.game_chat_window.text.text.reset_children = True

    self.game_window.list.text.reset_children = True
    self.board_window.list.text.reset_children = True
    self.my_server_window.list.text.reset_children = True
    self.user_window.list.text.reset_children = True
    
    self.server_chat_window.on_resize()
    self.room_chat_window.on_resize()
    self.game_chat_window.on_resize()

    self.game_window.on_resize()
    self.board_window.on_resize()
    self.my_server_window.on_resize()
    self.user_window.on_resize()

    self.server_chat_window.text.update_scrollbar()
    self.room_chat_window.text.update_scrollbar()
    self.game_chat_window.text.update_scrollbar()

    self.game_window.list.update_scrollbar()
    self.board_window.list.update_scrollbar()
    self.my_server_window.list.update_scrollbar()
    self.user_window.list.update_scrollbar()

  def stop(self) :
    if self.zone_net.sock :
      self.zone_net.sock.close()
    states.ApplicationState.stop(self)
