import sys

import soya
from soya import pudding

import states
import widgets

class BooleanOption(pudding.container.HorizontalContainer):
  class Label(pudding.control.SimpleLabel):
    def __init__(self, label = '', **kwargs):
      pudding.control.SimpleLabel.__init__(self, label = label, **kwargs)

      self.color = pudding.STYLE.BLUE1

  class Button(widgets.ImageButton):
    pass 
  
  def __init__(self, parent = None, label = '', value = False, callback = None, **kwargs):
    kwargs['height'] = 50
    pudding.container.HorizontalContainer.__init__(self, parent, **kwargs)

    self.label = self.Label(label = label)

    self.add_child(self.label, pudding.ALIGN_LEFT)
    
    self.tick = soya.Material.get('tick')
    self.cross = soya.Material.get('cross')

    self.value = value
    self.callback = callback

    if value:
      m = self.tick
    else:
      m = self.cross
    
    self.image = self.Button(func = self.change, material = m, width = 50, height = 50)
    self.add_child(self.image, pudding.ALIGN_RIGHT)

  def on_resize(self):
    self.width = self.parent.width - 20
    pudding.container.HorizontalContainer.on_resize(self)
    self.image.set_pos_bottom_right(right = 0)

  def change(self):
    self.value = not self.value

    if self.value:
      m = self.tick
    else:
      m = self.cross

    self.image.material = m

    if self.callback:
      self.callback(self.value)

class ConfigMenuState(states.BaseMenuState):
  """ State for the Main menu """
  def get_menu_options(self):
    return [
          #(BooleanOption(label = 'Fullscreen', 
          #                callback = self.toggle_fullscreen,
          #                value = self.application.config.getboolean('video', 'fullscreen'),
          #                ), pudding.EXPAND_HORIZ), 
          self.MenuOption('Reset GNUZone info', self.reset_gnuz),
          self.MenuOption('Back', self.do_back),
          ]
        
  def do_back(self):
    self.application.set_state(states.MainMenuState)

  def reset_gnuz(self):
    prompt = widgets.YesNoPrompt(text = 'Are you sure you want to reset your GNUZone infomation?', 
                                  callback = self.really_reset_gnuz,
                                  width = 300,
                                  height = 200)


  def really_reset_gnuz(self, prompt):
    if prompt.value:
      self.application.config.remove_section('gnuzone')
      
      widgets.MessagePrompt(text = 'GNUZone information reset',
                            width = 300,
                            height = 200,
                            )

    else:
      widgets.MessagePrompt(text = 'GNUZone information not reset',
                            width = 300,
                            height = 200,
                            )

  def toggle_fullscreen(self, value):
    self.application.config.set('video', 'fullscreen', value)
    self.application.config.save()

    if value: 
      fs = 1
    else:
      fs = 0 
      
    soya.set_video(self.root_widget.width, self.root_widget.height, fs, 0)


