import sys

import soya
from soya import pudding
from soya.opengl import *
from soya import sdlconst as sdl

import states
import widgets
import board

class Container(pudding.container.VerticalContainer):
  def render_self(self):
    pudding.container.VerticalContainer.render_self(self)

    glTranslatef(-20, -10, 0)
    pudding.STYLE.draw_bordered_box(self.width, self.height)


class Label(pudding.control.Label):
  def __init__(self, parent = None, func = None, **kwargs):
    pudding.control.Label.__init__(self, parent, **kwargs)
    self.font = pudding.STYLE.small_font
    self.color = pudding.STYLE.BLUE1
    self.func = func

  def on_focus(self):
    if self.func:
      self.color = (0.0, 0.0, 0.0, 1.0)
    
  def on_loose_focus(self):
    self.color = pudding.STYLE.BLUE1
  
  def on_mouse_up(self, x, y, buttons):
    if self.func:
      self.func()

    return True



class GameList(pudding.control.Panel):
  def __init__(self, parent, **kwargs):
    pudding.control.Panel.__init__(self, parent, **kwargs)

    c = pudding.core.Container(self, left = 0,
                                      top = 0,
                                      right = 0,
                                      bottom = 0,
                                     )

    self.list = widgets.ScrollingBuffer(c, left = 10,
                                        top = 10,
                                        right = 10,
                                        bottom = 10,
                                        buffer_class = widgets.SelectableScrollingBufferText,
                                        )

    self.label = pudding.control.Label(c, left = 20,
                                    top = 10,
                                    width = c.width,
                                    height = 20,
                                    label = 'Board Select:',
                                    )
    self.label.font = pudding.STYLE.small_font
    self.label.color = pudding.STYLE.BLUE1

  def on_resize(self):
    pudding.control.Panel.on_resize(self)
    self.list.resize(0, 0, self.width, self.height)


class RootWidget(widgets.RootWidget):
  def on_resize(self):
    widgets.RootWidget.on_resize(self)
    self.state.on_resize()

  def on_key_down(self, key, mods):
    if key == sdl.K_ESCAPE:
      self.state.application.set_state(states.MainMenuState)
      return True

class LocalMenuState(states.BaseMenuState):
  """ State for the Main menu """

  ROOT_WIDGET_CLASS = RootWidget

  def __init__(self, application):
    self.widgets_ok = False
    states.BaseMenuState.__init__(self, application)
  
  def setup_scene(self):
    states.BaseMenuState.setup_scene(self)
    self.menu.height = 50

    """
    self.players_list = Container(self.root_widget,
                                    left = 100,
                                    top = 150,
                                    width = 300,
                                    height = 200,
                                    )

    msg = Label(label = 'AI players:')
    msg.height = 70
    self.one_player = Label(label = '  1   ', func = self.click_one_player,)
    self.two_player = Label(label = '  2   ', func = self.click_two_player,)
    self.player_selection = self.one_player

    self.marker = pudding.control.Image(material = soya.Material.get('scroll1'),
                                    width = 20,
                                    height = 20)

    self.players_list.add_child(msg, pudding.EXPAND_HORIZ)
    self.players_list.add_child(self.one_player)
    self.players_list.add_child(self.two_player)
    self.players_list.add_child(self.marker)

    self.move_marker()
    """

    self.player_list = GameList(self.root_widget,
                                    left = 100,
                                    top = 140,
                                    width = 300,
                                    height = 200,
                                    )

    self.player_list.label.label = "Players:"
    self.player_list.list.extend(['1', '2'])

    self.game_list = GameList(self.root_widget,
                                    left = 400,
                                    top = 140,
                                    width = 300,
                                    height = 200,
                                    )

    self.game_list.list.extend( board.get_board_files())

    self.go_but = widgets.Button(self.root_widget,
                                    left = 390,
                                    top = self.game_list.top + self.game_list.height + 10,
                                    label = 'Play',
                                    func = self.do_play)

    self.widgets_ok = True
  
  def get_menu_options(self):
    return [
          self.MenuOption('Back', self.do_back),
          ]
       
  def do_play(self):
    sel = self.game_list.list.text.selection
    
    if not sel:
      widgets.MessagePrompt(text = 'Please select a board', width = 300, height = 200)
      return 

    board_name = sel.label 

    sel = self.player_list.list.text.selection

    if not sel:
      widgets.MessagePrompt(text = 'Please select number of players', width = 300, height = 200)
      return 
    
    players = int(sel.label)

    self.application.set_state(states.GameState, 
                                local = True, 
                                nrplayers = players,
                                board_name = board_name,
                                )
       
  def do_back(self):
    self.application.set_state(states.MainMenuState)

  def click_one_player(self):
    self.player_selection = self.one_player
    self.move_marker()

  def click_two_player(self):
    self.player_selection = self.two_player
    self.move_marker()
  
  def on_resize(self):
    if not self.widgets_ok:
      return 
    
    self.move_marker()

    self.game_list.list.text.reset_children = True
    self.game_list.on_resize()
    self.game_list.list.update_scrollbar()

    self.player_list.list.text.reset_children = True
    self.player_list.on_resize()
    self.player_list.list.update_scrollbar()

  def move_marker(self):
    #self.marker.left = 100
    #self.marker.top = self.player_selection.top
    pass


