import sys

from soya import sdlconst as sdl

import states
import widgets

class RootWidget(widgets.RootWidget):
  def on_key_down(self, key, mods):
    if key == sdl.K_ESCAPE:
      self.state.try_exit()
      

class MainMenuState(states.BaseMenuState):
  """ State for the Main menu """
  
  ROOT_WIDGET_CLASS = RootWidget
  
  def start(self):
    self.application.play_effect('interface/state_change.wav')
    states.BaseMenuState.start(self)
  
  def get_menu_options(self):
    return [
          self.MenuOption('Help', self.do_help),
          self.MenuOption('Local Game', self.do_local),
#          self.MenuOption('Network Game', self.do_network),
          #self.MenuOption('Options', self.do_options),
          self.MenuOption('Quit', self.try_exit),
          self.MenuOption('Credits', self.do_credits),
          #self.MenuOption('Editor (broken)', self.debug_editor),
          ]
        
  def do_local(self):
    self.application.set_state(states.LocalMenuState)
  
  def do_network(self):
    self.application.set_state(states.KioskState)
  
  def do_options(self):
    self.application.set_state(states.ConfigMenuState)
 
  def do_credits(self):
    self.application.set_state(states.IntroState)

  def do_help(self):
    self.application.set_state(states.HelpState)

  def debug_game(self):
    self.application.set_state(states.GameState)

  def debug_editor(self):
    self.application.set_state(states.EditorState)

  def try_exit(self):
    prompt = widgets.YesNoPrompt(text = 'Do you really want to quit?', 
                              callback = self.exit_callback,
                              width = 400,
                              height = 150,
                              )

  def exit_callback(self, prompt):
    if prompt.value:
      sys.exit()

