import soya
from soya import pudding
from soya.pudding.style import Style as BaseStyle
from soya.pudding.sysfont import SysFont
from soya.opengl import *

class Style(BaseStyle):
  def __init__(self):
    BaseStyle.__init__(self)

    self.default_font = soya.Font(SysFont('Arail, sans, freesans'), 24, 20)
    self.small_font = soya.Font(SysFont('Arail, sans, freesans'), 20, 15)
    self.prompt_font = self.small_font
    self.panel_font = self.small_font

    self.gui_mat = soya.Material.get('gui_stuff')
    self.gui_mat.clamp = 1

    self.GREEN1 = (0.68627450980392157, 0.8784313725490196, 0.87450980392156863, 1.0)
    self.BLUE1 = (0.11372549019607843, 0.17254901960784313, 0.56862745098039214, 1.0)
    self.BLUE2 = (0.11372549019607843, 0.17254901960784313, 0.56862745098039214, 0.5)

  def draw_panel(self, width, height, background = None, border = None,
                      border_width = None, label = ''):

    self.draw_bordered_box(width, height, background, border, border_width, decoration = True)

  
  def draw_bordered_box(self, width, height, background = None, border = None,
                        border_width = None, decoration = False, tab = False):

    if height < 40:
      BaseStyle.draw_bordered_box(self,width, height, background, border, border_width)
      return 
    
    self.gui_mat.activate()

    glEnable(GL_BLEND)
    glBegin(GL_QUADS)

    s = 1/512.
    c = 20
    cs = s * c     

    u1, v1 = 0, 0
    u2, v2 = u1 + cs, v1 + cs 
    
    glTexCoord2f(u1, v1)
    glVertex3f(0, 0, 0)
    glTexCoord2f(u1, v2)
    glVertex3f(0, c, 0)
    glTexCoord2f(u2, v2)
    glVertex3f(c, c, 0)
    glTexCoord2f(u2, v1)
    glVertex3f(c, 0, 0)

    u1, u2 = u2, u1 

    #u1, v1 = 0.25, 0
    #u2 = 0.333984375
   
    glTexCoord2f(u1, v1)
    glVertex3f(width - c, 0, 0)
    glTexCoord2f(u1, v2)
    glVertex3f(width - c, c, 0)
    glTexCoord2f(u2, v2)
    glVertex3f(width, c, 0)
    glTexCoord2f(u2, v1)
    glVertex3f(width, 0, 0)

    if not tab:
      u1, u2 = u2, u1 
      v1, v2 = v2, v1
      
      glTexCoord2f(u1, v1)
      glVertex3f(0, height - c, 0)
      glTexCoord2f(u1, v2)
      glVertex3f(0, height, 0)
      glTexCoord2f(u2, v2)
      glVertex3f(c, height, 0)
      glTexCoord2f(u2, v1)
      glVertex3f(c, height - c, 0)

      u1, u2 = u2, u1 
      
      glTexCoord2f(u1, v1)
      glVertex3f(width - c, height - c, 0)
      glTexCoord2f(u1, v2)
      glVertex3f(width - c, height, 0)
      glTexCoord2f(u2, v2)
      glVertex3f(width, height, 0)
      glTexCoord2f(u2, v1)
      glVertex3f(width, height - c, 0)
    
    u1, v1 = 0.26, 0.25
    u2, v2 = u1 + 0.0058125, v1 + cs

    glTexCoord2f(u1, v1)
    glVertex3f(c, 0, 0)
    glTexCoord2f(u1, v2)
    glVertex3f(c, c, 0)
    glTexCoord2f(u2, v2)
    glVertex3f(width - c, c, 0)
    glTexCoord2f(u2, v1)
    glVertex3f(width - c, 0, 0)

    if not tab:
      glTexCoord2f(u1, v2)
      glVertex3f(c, height - c, 0)
      glTexCoord2f(u1, v1)
      glVertex3f(c, height, 0)
      glTexCoord2f(u2, v1)
      glVertex3f(width - c, height, 0)
      glTexCoord2f(u2, v2)
      glVertex3f(width - c, height - c, 0)

    u1, v1 = 0.5, 0.251
    u2, v2 = u1 + cs, v1 + 0.0058125

    if tab:
      h = height - 2
    else:
      h = height - c
    
    glTexCoord2f(u1, v1)
    glVertex3f(0, c, 0)
    glTexCoord2f(u1, v2)
    glVertex3f(0, h, 0)
    glTexCoord2f(u2, v2)
    glVertex3f(c, h, 0)
    glTexCoord2f(u2, v1)
    glVertex3f(c, c, 0)

    u1, u2 = u2, u1
    
    glTexCoord2f(u1, v1)
    glVertex3f(width - c, c, 0)
    glTexCoord2f(u1, v2)
    glVertex3f(width - c, h, 0)
    glTexCoord2f(u2, v2)
    glVertex3f(width, h, 0)
    glTexCoord2f(u2, v1)
    glVertex3f(width, c, 0)

    # TODO do window decoration image here too!

    glEnd()

    soya.DEFAULT_MATERIAL.activate()

    glColor4f(0.11372549019607843, 0.17254901960784313, 0.56862745098039214, 0.25098039215686274)

    glBegin(GL_QUADS)
    glVertex3f(c, c, 0)
    glVertex3f(c, h, 0)
    glVertex3f(width - c, h, 0)
    glVertex3f(width - c, c, 0)
    glEnd()
    
    glDisable(GL_BLEND)

